<?php 
/************************************
* Pagination Object 
* Created 12.28.05 - C.Paulson
* Modified 12.28.05 - C.Paulson
*************************************/

class Paginate {
	
	// for the outside world to view
	var $num_pages;
	var $active_page;
	var $start_record;
	var $end_record;
	var $num_records;
	var $offset;
	var $url;
	var $per_page;

	/*******************
	* RETURN: VOID
	*
	* Constructor Class
	********************/
	function Paginate(){
	
	
	}
	
	/***************************
	* RETURN: String
	*
	* Build Pagination Link Row
	****************************/
	function buildPagination($total,$per_page,$offset,$url){
	
		// keep things in check for security
		if($offset > $total){
			$offset = $total - $per_page;
		} elseif($offset < 0){
			$offset = 0;
		}
		// get number of pages
		$this->num_pages = $this->numPages($total,$per_page);
		// get active page
		$this->active_page = $this->activePage($offset,$per_page);
		// set number of records
		$this->num_records = $total;
		// set first record
		$this->start_record = $offset + 1;
		// set end record
		$tmp_end = $offset + $per_page;
		if($tmp_end > $total){
			$this->end_record = $total;
		} else {
			$this->end_record = $tmp_end;
		}
		// set offset
		$this->offset = $offset;
		// set url
		$this->url = $url;
		// set # record per page
		$this->per_page = $per_page;
	
		// set page start and create counter
		$page_start = $this->pageStart($this->active_page);
		$i = $page_start;
		// determine group end
		$page_end = $this->pageEnd($page_start,$this->num_pages);

		// begin output
		if($i > 5){
			$first_url = $this->parseLink($url,0);
			$p_group = $this->previousGroup($i,$per_page,$url);
			$output = "<a href=\"".$first_url."\">1</a>".$p_group."";
		} else {
			$output = "";
		}
		
		// build pagination link row
		while($i <= $page_end){
		
			if($i == $this->active_page){
		
				// bold current page
				$output .= "<strong>".$this->active_page."</strong> ";
				
			} else {

				// get offset for current page
				$this_offset = $this->getOffset($i,$per_page);
				// get url for current page
				$this_url = $this->parseLink($url,$this_offset);
				
				// link different page
				$output .= "<a href=\"".$this_url."\">".$i."</a> ";
				
			}
				
			$i++;	
		
		}
		
		// determine if we have more groups in front of us
		if($this->num_pages > $page_end){
			
			// create next group link
			$n_group = $this->nextGroup($page_start,$per_page,$url);
			$output .= $n_group."";
		
		}
		
		// append "of num_pages"
		if($this->num_pages == $this->active_page){
		
			$output .= " of ".$this->num_pages;
		
		} else {
		
			$final_offset = $this->getOffset($this->num_pages,$per_page);
			$final_url = $this->parseLink($url,$final_offset);
			$output .=" of <a href=\"".$final_url."\">".$this->num_pages."</a>";
		
		}
		
		return $output;
	
	}
	
	/************************************************
	* RETURN: String
	*
	* Lists Record Numbers User Is Currently Viewing
	*************************************************/
	function showRecords(){
	
		$output = "Viewing ".$this->start_record." - ".$this->end_record." of ".$this->num_records;
		return $output;
	
	}
	
	/****************************************
	* RETURN: String
	*
	* Lists Next / Previous Pagination Links
	*****************************************/
	function showNext(){
	
		// allocate variables
		$previous = "";
		$next = "";
		$div = "";
	
		// create previous link
		if($this->offset != 0){
	
			$prev_offset = $this->offset - $this->per_page;
			$prev_url = $this->parseLink($this->url,$prev_offset);
			$previous .= "&laquo; <a href=\"".$prev_url."\">Previous</a>";
	
		}
		
		// create next link
		if($this->end_record != $this->num_records){

			$next_offset = $this->offset + $this->per_page;
			$next_url = $this->parseLink($this->url,$next_offset);
			$next .= "<a href=\"".$next_url."\">Next</a> &raquo;";		
		
		}
		
		// determine if we need a divider
		if(strlen($previous) && strlen($next)){
		
			$div = "|";
		
		}
		
		// construct links
		$output = $previous." ".$div." ".$next;
		
		return $output;
	
	}
	
	/**********************************
	* RETURN: String
	*
	* Build Individual Pagination Link
	***********************************/
	function parseLink($url,$offset){
	
		$output = preg_replace("/{OFFSET}/",$offset,$url);
		return $output;
	
	}
	
	/*****************************************
	* RETURN: Integer
	*
	* Determine The Numer Of Pages Given Total 
	******************************************/
	function numPages($total,$per_page){
	
		$num_pages = ceil($total / $per_page);
		return $num_pages;
		
	}
	
	/***************************
	* RETURN: Integer
	*
	* Determine the Active Page
	****************************/
	function activePage($offset,$per_page){
	
		if($offset == 0){
			return 1;
		} else {
			$active_page = ceil($offset / $per_page);
			$active_page++;
			return $active_page;
		}
	
	}
	
	/****************************************
	* RETURN: Integer
	*
	* Determine when pagination should begin
	*****************************************/
	function pageStart($active_page){
	
		// get length of digit
		$length = strlen($active_page);
	
		if($length > 1){
		
			// get the last digit
			$last = substr($active_page, -1, 1);
			// create new length for substr glue
			$new_length = $length - 1;
			// get the first X digits of our integer
			$first = substr($active_page, 0, $new_length);
			
			if($last == 0){
				return $active_page - 4;
			} elseif($last <= 5){
				return $first."1";
			} elseif($last > 5) {
				return $first."6";
			}
		
		} else {
		
			if($active_page <= 5){
				return 1;
			} else {
				return 6;
			}
		
		}
	
	}
	
	/********************************************************
	* RETURN: Integer
	*
	* Determine when pagination should end for current group
	*********************************************************/
	function pageEnd($active_page,$num_pages){
	
		$new_end = $active_page + 4;
	
		// test if we have overspill
		if($new_end <= $num_pages){
			return $new_end;
		} else {
			return $num_pages;
		}
	
	}
	
	/*******************************
	* RETURN: Integer
	*
	* Get Offset Of Particular Page
	********************************/
	function getOffset($current_page,$per_page){
	
		// decrease to ensure proper calculation
		$current_page = $current_page - 1;
	
		$offset = $current_page * $per_page;
		return $offset;
	
	}
	
	/*******************************************************
	* RETURN: String
	*
	* Dynamically create link for previous pagination group
	********************************************************/
	function previousGroup($start_page,$per_page,$url){
	
		// create new start page
		$new_start = $start_page - 5;
		// get new offset
		$new_offset = $this->getOffset($new_start,$per_page);
		// get new url
		$new_url = $this->parseLink($url,$new_offset);
		
		// generate output
		$output = "<a href=\"".$new_url."\"></a>";
	
		return $output;
	
	}

	/***************************************************
	* RETURN: String
	*
	* Dynamically create link for next pagination group
	****************************************************/
	function nextGroup($start_page,$per_page,$url){
	
		// create new start page
		$new_start = $start_page + 5;
		// get new offset
		$new_offset = $this->getOffset($new_start,$per_page);
		// get new url
		$new_url = $this->parseLink($url,$new_offset);
		
		// generate output
		$output = "<a href=\"".$new_url."\"></a>";
	
		return $output;
	
	}
	
}
?>